@@ -1,16 +1,15 @@
 import pickle
 from abc import ABCMeta, abstractmethod
 from collections import UserDict
-from datetime import datetime
 from logging import getLogger
-from typing import AsyncIterable, Callable, Iterable, Optional, Union
+from typing import AsyncIterable, Callable, Iterable, Optional, Tuple, Union
 
 from aiohttp import ClientResponse
 from aiohttp.typedefs import StrOrURL
 
+from aiohttp_client_cache.cache_control import CacheActions, ExpirationPatterns, ExpirationTime
 from aiohttp_client_cache.cache_keys import create_key
 from aiohttp_client_cache.docs.forge_utils import extend_init_signature
-from aiohttp_client_cache.expiration import ExpirationPatterns, ExpirationTime, get_expiration
 from aiohttp_client_cache.response import AnyResponse, CachedResponse
 
 ResponseOrKey = Union[CachedResponse, bytes, str, None]
@@ -23,7 +22,6 @@ class CacheBackend:
     This manages higher-level cache operations, including cache expiration, generating cache keys,
     and managing redirect history.
 
-
     Lower-level storage operations are handled by :py:class:`.BaseCache`.
     To extend this with your own custom backend, implement one or more subclasses of
     :py:class:`.BaseCache` to use as :py:attr:`CacheBackend.responses` and
@@ -39,19 +37,21 @@ class CacheBackend:
         allowed_methods: tuple = ('GET', 'HEAD'),
         include_headers: bool = False,
         ignored_params: Iterable = None,
+        cache_control: bool = False,
         filter_fn: Callable = lambda r: True,
         **kwargs,
     ):
         """
         Args:
             cache_name: Cache prefix or namespace, depending on backend
-            expire_after: Time after which a cache entry will be expired. For numeric values,
-                specify either a positive value in seconds, or ``-1`` to never expire.
+            expire_after: Time after which a cache entry will be expired; see
+                :ref:`user_guide:cache expiration` for possible formats
             urls_expire_after: Expiration times to apply for different URL patterns
             allowed_codes: Only cache responses with these status codes
             allowed_methods: Only cache requests with these HTTP methods
             include_headers: Cache requests with different headers separately
             ignored_params: Request parameters to be excluded from the cache key
+            cache_control: Use Cache-Control response headers
             filter_fn: function that takes a :py:class:`aiohttp.ClientResponse` object and
                 returns a boolean indicating whether or not that response should be cached. Will be
                 applied to both new and previously cached responses
@@ -61,6 +61,7 @@ class CacheBackend:
         self.urls_expire_after = urls_expire_after
         self.allowed_codes = allowed_codes
         self.allowed_methods = allowed_methods
+        self.cache_control = cache_control
         self.filter_fn = filter_fn
         self.disabled = False
 
@@ -71,28 +72,58 @@ class CacheBackend:
         self.include_headers = include_headers
         self.ignored_params = set(ignored_params or [])
 
-    def is_cacheable(self, response: Union[AnyResponse, None]) -> bool:
+    def is_cacheable(
+        self, response: Union[AnyResponse, None], actions: CacheActions = None
+    ) -> bool:
         """Perform all checks needed to determine if the given response should be cached"""
         if not response:
             return False
+
         cache_criteria = {
-            'allowed method': response.method in self.allowed_methods,
-            'allowed status': response.status in self.allowed_codes,
-            'not disabled': not self.disabled,
-            'not expired': not getattr(response, 'is_expired', False),
-            'not filtered': self.filter_fn(response),
+            'disabled cache': self.disabled,
+            'disabled method': str(response.method) not in self.allowed_methods,
+            'disabled status': response.status not in self.allowed_codes,
+            'disabled by filter': not self.filter_fn(response),
+            'disabled by headers or expiration params': actions and actions.skip_write,
+            'expired': getattr(response, 'is_expired', False),
         }
         logger.debug(f'Pre-cache checks for response from {response.url}: {cache_criteria}')  # type: ignore
-        return all(cache_criteria.values())
+        return not any(cache_criteria.values())
 
-    async def get_response(self, key: str) -> Optional[CachedResponse]:
-        """Retrieve response and timestamp for `key` if it's stored in cache,
-        otherwise returns ``None```
+    async def request(
+        self,
+        method: str,
+        url: StrOrURL,
+        expire_after: ExpirationTime = None,
+        **kwargs,
+    ) -> Tuple[Optional[CachedResponse], CacheActions]:
+        """Fetch a cached response based on request info
 
         Args:
-            key: key of resource
+            method: HTTP method
+            url: Request URL
+            expire_after: Expiration time to set only for this request; overrides
+                ``CachedSession.expire_after``, and accepts all the same values.
+            kwargs: All other request arguments
         """
-        # Attempt to fetch response from the cache
+        key = self.create_key(method, url, **kwargs)
+        actions = CacheActions.from_request(
+            key,
+            url=url,
+            request_expire_after=expire_after,
+            session_expire_after=self.expire_after,
+            urls_expire_after=self.urls_expire_after,
+            cache_control=self.cache_control,
+            **kwargs,
+        )
+
+        # Skip reading from the cache, if specified by request headers
+        response = None if actions.skip_read else await self.get_response(actions.key)
+        return response, actions
+
+    async def get_response(self, key: str) -> Optional[CachedResponse]:
+        """Fetch a cached response based on a cache key"""
+        # Attempt to fetch the cached response
         logger.debug(f'Attempting to get cached response for key: {key}')
         try:
             response = await self.responses.read(key) or await self._get_redirect_response(str(key))
@@ -103,11 +134,11 @@ class CacheBackend:
             logger.debug('No cached response found')
         # If the item is expired or filtered out, delete it from the cache
         elif not self.is_cacheable(response):  # type: ignore
-            logger.info('Cached response expired; deleting')
+            logger.debug('Cached response expired; deleting')
             response = None
             await self.delete(key)
         else:
-            logger.info(f'Cached response found for key: {key}')
+            logger.debug(f'Cached response found for key: {key}')
 
         # Response will be a CachedResponse or None by this point
         return response  # type: ignore
@@ -117,36 +148,25 @@ class CacheBackend:
         redirect_key = await self.redirects.read(key)
         return await self.responses.read(redirect_key) if redirect_key else None  # type: ignore
 
-    async def save_response(
-        self, key: str, response: ClientResponse, expire_after: ExpirationTime = None
-    ):
-        """Save response to cache
+    async def save_response(self, response: ClientResponse, actions: CacheActions):
+        """Save a response to the cache
 
         Args:
-            key: Key for this response
             response: Response to save
-            expire_after: Expiration time to set only for this request; overrides
-                ``CachedSession.expire_after``, and accepts all the same values.
+            actions: Specific cache actions to take
         """
-        if not self.is_cacheable(response):
+        actions.update_from_response(response)
+        if not self.is_cacheable(response, actions):
+            logger.debug(f'Not caching response for key: {actions.key}')
             return
-        logger.info(f'Saving response for key: {key}')
 
-        expire_after = self._get_expiration(response, expire_after)
-        cached_response = await CachedResponse.from_client_response(response, expire_after)
-        await self.responses.write(key, cached_response)
+        logger.debug(f'Saving response for key: {actions.key}')
+        cached_response = await CachedResponse.from_client_response(response, actions.expires)
+        await self.responses.write(actions.key, cached_response)
 
         # Alias any redirect requests to the same cache key
         for r in response.history:
-            await self.redirects.write(self.create_key(r.method, r.url), key)
-
-    def _get_expiration(
-        self, response: ClientResponse, request_expire_after: ExpirationTime = None
-    ) -> Optional[datetime]:
-        """Get the appropriate expiration for the given response"""
-        return get_expiration(
-            response, request_expire_after, self.expire_after, self.urls_expire_after
-        )
+            await self.redirects.write(self.create_key(r.method, r.url), actions.key)
 
     async def clear(self):
         """Clear cache"""
@@ -180,28 +200,28 @@ class CacheBackend:
             if response and response.is_expired or not self.filter_fn(response):
                 keys_to_delete.add(key)
 
-        logger.info(f'Deleting {len(keys_to_delete)} expired cache entries')
+        logger.debug(f'Deleting {len(keys_to_delete)} expired cache entries')
         for key in keys_to_delete:
             await self.delete(key)
 
-    def create_key(self, *args, **kwargs):
+    def create_key(self, method: str, url: StrOrURL, **kwargs):
         """Create a unique cache key based on request details"""
         return create_key(
-            *args,
+            method,
+            url,
             include_headers=self.include_headers,
             ignored_params=self.ignored_params,
             **kwargs,
         )
 
-    async def delete_url(self, url: StrOrURL):
-        """Delete cached response associated with `url`, along with its history (if applicable).
-        Works only for GET requests.
-        """
-        await self.delete(self.create_key('GET', url))
+    async def delete_url(self, url: StrOrURL, method: str = 'GET', **kwargs):
+        """Delete cached response associated with `url`, along with its history (if applicable)"""
+        key = self.create_key(url=url, method=method, **kwargs)
+        await self.delete(key)
 
-    async def has_url(self, url: StrOrURL) -> bool:
-        """Returns `True` if cache has `url`, `False` otherwise. Works only for GET request urls"""
-        key = self.create_key('GET', url)
+    async def has_url(self, url: StrOrURL, method: str = 'GET', **kwargs) -> bool:
+        """Returns `True` if cache has `url`, `False` otherwise"""
+        key = self.create_key(method=method, url=url, **kwargs)
         return await self.responses.contains(str(key)) or await self.redirects.contains(str(key))
 
     async def get_urls(self) -> AsyncIterable[str]:
@@ -209,8 +229,12 @@ class CacheBackend:
         async for r in self.responses.values():
             yield r.url  # type: ignore
 
+    async def close(self):
+        """Close any active connections, if applicable"""
+
 
 # TODO: Support yarl.URL like aiohttp does?
+# TODO: Implement __aiter__?
 class BaseCache(metaclass=ABCMeta):
     """A wrapper for lower-level cache storage operations. This is separate from
     :py:class:`.CacheBackend` to allow a single backend to contain multiple cache objects.
